//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: BLOB.CPP $
//  $Date: 1995/07/14 16:17:34 $


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructors.

TDEWBlob::TDEWBlob()
  : fBuffer(0),
    fHCursor(NULL)
{
}

TDEWBlob::TDEWBlob(TDEWRecord& record, const UINT16 iField, const DBIOpenMode eOpenMode)
  : fBuffer(record.fBuffer),
    fField(iField),
    fHCursor(record.Cursor().Handle())
{
    record.CheckFieldNumber(iField);

    TDEWException::CheckResult(DbiOpenBlob(fHCursor, fBuffer, fField, eOpenMode));
}

TDEWBlob::TDEWBlob(TDEWRecord& record, const string& fieldName, const DBIOpenMode eOpenMode)
  : fBuffer(record.fBuffer),
    fField(record.FieldNumber(fieldName)),
    fHCursor(record.Cursor().Handle())
{
    TDEWException::CheckResult(DbiOpenBlob(fHCursor, fBuffer, fField, eOpenMode));
}


// Destructor.

TDEWBlob::~TDEWBlob()
{
    if (fBuffer != 0)
        Close();
}


// Close the blob.

void TDEWBlob::Close()
{
    TDEWException::CheckResult(DbiFreeBlob(fHCursor, fBuffer, fField));
    fBuffer = 0;
    fHCursor = NULL;
}


// Read bytes from blob.

UINT32 TDEWBlob::Get(const UINT32 iOffset, const UINT32 iLen, const pBYTE pDest) const
{
    UINT32 iRead;

    TDEWException::CheckResult(DbiGetBlob(fHCursor, fBuffer, fField, iOffset, iLen, pDest, &iRead));

    return iRead;
}


// Get blob heading.

void TDEWBlob::GetHeading(const pBYTE pDest) const
{
    TDEWException::CheckResult(DbiGetBlobHeading(fHCursor, fField, fBuffer, pDest));
}


// Open blob.

void TDEWBlob::Open(TDEWRecord& record, const UINT16 iField, const DBIOpenMode eOpenMode)
{
    if (fBuffer != 0)
        Close();

    record.CheckFieldNumber(iField);

    fHCursor = record.Cursor().Handle();
    fBuffer = record.fBuffer;
    fField = iField;

    TDEWException::CheckResult(DbiOpenBlob(fHCursor, fBuffer, fField, eOpenMode));
}

void TDEWBlob::Open(TDEWRecord& record, const string& fieldName, const DBIOpenMode eOpenMode)
{
    if (fBuffer != 0)
        Close();

    fHCursor = record.Cursor().Handle();
    fBuffer = record.fBuffer;
    fField = record.FieldNumber(fieldName);

    TDEWException::CheckResult(DbiOpenBlob(fHCursor, fBuffer, fField, eOpenMode));
}


// Write bytes to blob.

void TDEWBlob::Put(const UINT32 iOffset, const UINT32 iLen, const pcBYTE pSrc)
{
    TDEWException::CheckResult(DbiPutBlob(fHCursor, fBuffer, fField, iOffset, iLen, (pBYTE) pSrc));
}


// Get the size of a blob.

UINT32 TDEWBlob::Size() const
{
    UINT32 iSize;

    TDEWException::CheckResult(DbiGetBlobSize(fHCursor, fBuffer, fField, &iSize));

    return iSize;
}


// Reduce the blob size.

void TDEWBlob::Truncate(const UINT32 iLen)
{
    TDEWException::CheckResult(DbiTruncateBlob(fHCursor, fBuffer, fField, iLen));
}
