//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: CURSOR.CPP $
//  $Date: 1995/07/14 16:17:34 $


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructors.

TDEWCursor::TDEWCursor()
  : fHCursor(NULL),
    fHDatabase(NULL),
    fFieldDesc(0),
    fRecord(0)
{
}

TDEWCursor::TDEWCursor(const hDBICur hCursor, const BOOL hasFields)
  : fHCursor(hCursor),
    fFieldDesc(0),
    fRecord(0)
{
    if (fHCursor != NULL) {
        TDEWException::CheckResult(DbiGetObjFromObj(fHCursor, objDATABASE, &fHDatabase));
        GetProperties(hasFields);
    }
}

TDEWCursor::TDEWCursor(TDEWDatabase& database)
  : fHCursor(NULL),
    fHDatabase(database.Handle()),
    fFieldDesc(0),
    fRecord(0)
{
}

TDEWCursor::TDEWCursor(TDEWDatabase& database, const pcCHAR pszTableName, const pcCHAR pszDriverType, const DBIOpenMode eOpenMode, const DBIShareMode eShareMode, const pcCHAR pszIndexName, const pcCHAR pszIndexTagName, const UINT16 iIndexId, const XLTMode exltMode, const BOOL bUniDirectional, const pcBYTE pOptParams)
  : fHCursor(NULL),
    fHDatabase(database.Handle()),
    fFieldDesc(0),
    fRecord(0)
{
    Open(pszTableName, pszDriverType, eOpenMode, eShareMode, pszIndexName, pszIndexTagName, iIndexId, exltMode, bUniDirectional, pOptParams);
}

TDEWCursor::TDEWCursor(const TDEWCursor& cursor, const BOOL bReadOnly, const BOOL bUniDirectional)
  : fHCursor(NULL),
    fFieldDesc(0),
    fRecord(0)
{
    if (cursor.IsOpen()) {
        TDEWException::CheckResult(DbiGetObjFromObj(cursor.fHCursor, objDATABASE, &fHDatabase));
        TDEWException::CheckResult(DbiCloneCursor(cursor.fHCursor, bReadOnly, bUniDirectional, &fHCursor));
        GetProperties(cursor.fFieldDesc != 0);
    }
}


// Destructor.

TDEWCursor::~TDEWCursor()
{
    if (IsOpen())
        Close();
}


// Assignment operator.

TDEWCursor& TDEWCursor::operator=(const TDEWCursor& cursor)
{
    if (IsOpen())
        Close();

    if (cursor.IsOpen()) {
        TDEWException::CheckResult(DbiGetObjFromObj(cursor.fHCursor, objDATABASE, &fHDatabase));
        TDEWException::CheckResult(DbiCloneCursor(cursor.fHCursor, FALSE, FALSE, &fHCursor));
        GetProperties(cursor.fFieldDesc != 0);
    }

    return *this;
}


// Lock a table.

void TDEWCursor::AcqTableLock(const DBILockType eLockType)
{
    TDEWException::CheckResult(DbiAcqTableLock(fHCursor, eLockType));
}


// Add a new index.

void TDEWCursor::AddIndex(const IDXDesc& idxDesc, const pcCHAR pszKeyviolName)
{
    TDEWException::CheckResult(DbiAddIndex(fHDatabase, fHCursor, NULL, NULL, (pIDXDesc) &idxDesc, (pCHAR) pszKeyviolName));
}


// Find/get the next record.

BOOL TDEWCursor::Advance(const DBILockType eLock, const pRECProps pRecProps)
{
    const DBIResult result = DbiGetNextRecord(fHCursor, eLock, fRecord->fBuffer, pRecProps);

    if (result == DBIERR_NONE)
        return TRUE;
    else if (result == DBIERR_EOF)
        return FALSE;
    else
        throw TDEWException(result);
}


// Insert a new record.

void TDEWCursor::AppendRecord()
{
    TDEWException::CheckResult(DbiAppendRecord(fHCursor, fRecord->fBuffer));
}


// Convert cursor to a link cursor.

void TDEWCursor::BeginLinkMode()
{
    TDEWException::CheckResult(DbiBeginLinkMode(&fHCursor));
}


// Close the cursor.

void TDEWCursor::Close()
{
    delete fRecord;
    delete [] fFieldDesc;

    TDEWException::CheckResult(DbiCloseCursor(&fHCursor));
    fHCursor = NULL;
    fRecord = 0;
    fFieldDesc = 0;
}


// Close an index.

void TDEWCursor::CloseIndex(const pcCHAR pszIndexName, const UINT16 iIndexId)
{
    TDEWException::CheckResult(DbiCloseIndex(fHCursor, (pCHAR) pszIndexName, iIndexId));
}


// Compare two book-marks.

CMPBkMkRslt TDEWCursor::CompareBookMarks(const pcBYTE pBookMark1, const pcBYTE pBookMark2) const
{
    CMPBkMkRslt cmpBkMkRslt;

    TDEWException::CheckResult(DbiCompareBookMarks(fHCursor, (pBYTE) pBookMark1, (pBYTE) pBookMark2, &cmpBkMkRslt));

    return cmpBkMkRslt;
}


// Compare two keys.

INT16 TDEWCursor::CompareKeys(const pcBYTE pKey1, const pcBYTE pKey2, const UINT16 iFields, const UINT16 iLen) const
{
    INT16 result;

    TDEWException::CheckResult(DbiCompareKeys(fHCursor, (pBYTE) pKey1, (pBYTE) pKey2, iFields, iLen, &result));

    return result;
}


// Get cursor properties.

/*CURProps& TDEWCursor::CursorProps()
{
    return fCursorProps;
}*/


const CURProps& TDEWCursor::CursorProps() const
{
    return fCursorProps;
}


// Delete index.

void TDEWCursor::DeleteIndex(const pcCHAR pszIndexName, const UINT16 iIndexNum, const pcCHAR pszIndexTagName)
{
    TDEWException::CheckResult(DbiDeleteIndex(fHDatabase, fHCursor, NULL, NULL, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIndexNum));
}


// Delete the current record.

TDEWRecord& TDEWCursor::DeleteRecord()
{
    TDEWException::CheckResult(DbiDeleteRecord(fHCursor, fRecord->fBuffer));

    return *fRecord;
}


// Delete all records.

void TDEWCursor::EmptyTable()
{
    TDEWException::CheckResult(DbiEmptyTable(fHDatabase, fHCursor, NULL, NULL));
}


// Convert cursor back to normal cursor.

void TDEWCursor::EndLinkMode()
{
    TDEWException::CheckResult(DbiEndLinkMode(&fHCursor));
}


// Extract the key value for a record.

void TDEWCursor::ExtractKey(const pBYTE pKey, const TDEWRecord& record) const
{
    TDEWException::CheckResult(DbiExtractKey(fHCursor, (pBYTE) record.fBuffer, pKey));
}


// Get field descriptions.

/*pFLDDesc TDEWCursor::FieldDescs()
{
    return fFieldDesc;
}*/


pcFLDDesc TDEWCursor::FieldDescs() const
{
    return fFieldDesc;
}


// Force reread of buffers from disk.

const TDEWRecord& TDEWCursor::ForceReread() const
{
    TDEWException::CheckResult(DbiForceReread(fHCursor));

    return *fRecord;
}

// Get a book-mark.

void TDEWCursor::GetBookMark(const pBYTE pBookMark) const
{
    TDEWException::CheckResult(DbiGetBookMark(fHCursor, pBookMark));
}


// Get a callback function.

void TDEWCursor::GetCallBack(const CBType ecbType, UINT32& iClientData, UINT16& iCbBufLen, pVOID& pCbBuf, pfDBICallBack& pfCb) const
{
    TDEWException::CheckResult(DbiGetCallBack(fHCursor, ecbType, &iClientData, &iCbBufLen, &pCbBuf, &pfCb));
}


// Get index properties.

void TDEWCursor::GetIndexDesc(const UINT16 iIndexSeqNo, IDXDesc& idxDesc) const
{
    TDEWException::CheckResult(DbiGetIndexDesc(fHCursor, iIndexSeqNo, &idxDesc));
}


void TDEWCursor::GetIndexDesc(IDXDesc& idxDesc) const
{
    TDEWException::CheckResult(DbiGetIndexDescs(fHCursor, &idxDesc));
}


// Get index descriptor for given field.

void TDEWCursor::GetIndexForField(const UINT16 iFld, IDXDesc& idxDesc, const BOOL bProdTagOnly) const
{
    TDEWException::CheckResult(DbiGetIndexForField(fHCursor, iFld, bProdTagOnly, &idxDesc));
}


// Find/get the next record.

TDEWRecord& TDEWCursor::GetNextRecord(const DBILockType eLock, const pRECProps pRecProps)
{
    TDEWException::CheckResult(DbiGetNextRecord(fHCursor, eLock, fRecord->fBuffer, pRecProps));

    return *fRecord;
}


// Find/get the prior record.

TDEWRecord& TDEWCursor::GetPriorRecord(const DBILockType eLock, const pRECProps pRecProps)
{
    TDEWException::CheckResult(DbiGetPriorRecord(fHCursor, eLock, fRecord->fBuffer, pRecProps));

    return *fRecord;
}


// Get the cursor properties.

void TDEWCursor::GetProperties(const BOOL hasFields)
{
    TDEWException::CheckResult(DbiGetCursorProps(fHCursor, &fCursorProps));

    if (hasFields) {
        fFieldDesc = new FLDDesc[fCursorProps.iFields];
        TDEWException::CheckResult(DbiGetFieldDescs(fHCursor, fFieldDesc));
    }

    fRecord = new TDEWRecord(*this);
}


// Get the current record.

TDEWRecord& TDEWCursor::GetRecord(const DBILockType eLock, const pRECProps pRecProps)
{
    TDEWException::CheckResult(DbiGetRecord(fHCursor, eLock, fRecord->fBuffer, pRecProps));

    return *fRecord;
}


// Find a record matching key.

TDEWRecord& TDEWCursor::GetRecordForKey(const TDEWRecord& record, const UINT16 iFields, const UINT16 iLen)
{
    TDEWException::CheckResult(DbiGetRecordForKey(fHCursor, FALSE, iFields, iLen, record.fBuffer, fRecord->fBuffer));

    return *fRecord;
}

TDEWRecord& TDEWCursor::GetRecordForKey(const pcBYTE pKey, const UINT16 iFields, const UINT16 iLen)
{
    TDEWException::CheckResult(DbiGetRecordForKey(fHCursor, TRUE, iFields, iLen, (pBYTE) pKey, fRecord->fBuffer));

    return *fRecord;
}


// Find/get a record by record number.

TDEWRecord& TDEWCursor::GetRelativeRecord(const INT32 iPosOffset, const DBILockType eLock, const pRECProps pRecProps)
{
    TDEWException::CheckResult(DbiGetRelativeRecord(fHCursor, iPosOffset, eLock, fRecord->fBuffer, pRecProps));

    return *fRecord;
}


// Get referential integrity descriptor.

void TDEWCursor::GetRintDesc(const UINT16 iRintSeqNo, RINTDesc &rintDesc) const
{
    TDEWException::CheckResult(DbiGetRintDesc(fHCursor, iRintSeqNo, &rintDesc));
}


// Get validity check descriptor.

void TDEWCursor::GetVchkDesc(const UINT16 iValSeqNo, VCHKDesc& valDesc) const
{
    TDEWException::CheckResult(DbiGetVchkDesc(fHCursor, iValSeqNo, &valDesc));
}


// Get index sequence number.

UINT16 TDEWCursor::IndexSeqNo(const pcCHAR pszIndexName, const UINT16 iIndexId, const pcCHAR pszTagName) const
{
    UINT16 iIndexSeqNo;

    TDEWException::CheckResult(DbiGetIndexSeqNo(fHCursor, (pCHAR) pszIndexName, (pCHAR) pszTagName, iIndexId, &iIndexSeqNo));

    return iIndexSeqNo;
}


// Initialize record area.

TDEWRecord& TDEWCursor::InitRecord()
{
    TDEWException::CheckResult(DbiInitRecord(fHCursor, fRecord->fBuffer));

    return *fRecord;
}


// Insert a new record.

TDEWRecord& TDEWCursor::InsertRecord(const DBILockType eLock)
{
    TDEWException::CheckResult(DbiInsertRecord(fHCursor, eLock, fRecord->fBuffer));

    return *fRecord;
}


// Check if current record is locked.

BOOL TDEWCursor::IsRecordLocked() const
{
    BOOL bLocked;

    TDEWException::CheckResult(DbiIsRecordLocked(fHCursor, &bLocked));

    return bLocked;
}


// Verify if table is locked.

UINT16 TDEWCursor::IsTableLocked(const DBILockType eLockType) const
{
    UINT16 iLocks;

    TDEWException::CheckResult(DbiIsTableLocked(fHCursor, eLockType, &iLocks));

    return iLocks;
}


// Verify if table is shared.

BOOL TDEWCursor::IsTableShared() const
{
    BOOL bShared;

    TDEWException::CheckResult(DbiIsTableShared(fHCursor, &bShared));

    return bShared;
}


// Link detail to master.

void TDEWCursor::LinkDetail(TDEWCursor& master, const UINT16 iLnkFields, const pcUINT16 piMstrFields, const pcUINT16 piDetlFields)
{
    TDEWException::CheckResult(DbiLinkDetail(master.fHCursor, fHCursor, iLnkFields, (pUINT16) piMstrFields, (pUINT16) piDetlFields));
}


void TDEWCursor::LinkDetail(TDEWCursor& master, const UINT16 iKeyLen, const pcCHAR pszMstrExp)
{
    TDEWException::CheckResult(DbiLinkDetailToExp(master.fHCursor, fHCursor, iKeyLen, (pCHAR) pszMstrExp));
}


// Get a list of locks.

TDEWCursor TDEWCursor::LockList(const BOOL bAllUsers, const BOOL bAllLockTypes) const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenLockList(fHCursor, bAllUsers, bAllLockTypes, &hCursor));

    return TDEWCursor(hCursor);
}


// Make temporary table permanent.

void TDEWCursor::MakePermanent(const pcCHAR pszName, const BOOL bOverWrite)
{
    TDEWException::CheckResult(DbiMakePermanent(fHCursor, (pCHAR) pszName, bOverWrite));
}


// Update the current record.

TDEWRecord& TDEWCursor::ModifyRecord(const BOOL bFreeLock)
{
    TDEWException::CheckResult(DbiModifyRecord(fHCursor, fRecord->fBuffer, bFreeLock));

    return *fRecord;
}


// Open a table.

void TDEWCursor::Open(TDEWDatabase& database, const pcCHAR pszTableName, const pcCHAR pszDriverType, const DBIOpenMode eOpenMode, const DBIShareMode eShareMode, const pcCHAR pszIndexName, const pcCHAR pszIndexTagName, const UINT16 iIndexId, const XLTMode exltMode, const BOOL bUniDirectional, const pcBYTE pOptParams)
{
    if (IsOpen())
        Close();

    fHDatabase = database.Handle();

    TDEWException::CheckResult(DbiOpenTable(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIndexId, eOpenMode, eShareMode, exltMode, bUniDirectional, (pBYTE) pOptParams, &fHCursor));

    GetProperties();
}

void TDEWCursor::Open(const pcCHAR pszTableName, const pcCHAR pszDriverType, const DBIOpenMode eOpenMode, const DBIShareMode eShareMode, const pcCHAR pszIndexName, const pcCHAR pszIndexTagName, const UINT16 iIndexId, const XLTMode exltMode, const BOOL bUniDirectional, const pcBYTE pOptParams)
{
    if (IsOpen())
        Close();

    TDEWException::CheckResult(DbiOpenTable(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIndexId, eOpenMode, eShareMode, exltMode, bUniDirectional, (pBYTE) pOptParams, &fHCursor));

    GetProperties();
}


// Open an index.

void TDEWCursor::OpenIndex(const pcCHAR pszIndexName, const UINT16 iIndexId)
{
    TDEWException::CheckResult(DbiOpenIndex(fHCursor, (pCHAR) pszIndexName, iIndexId));
}


// Pack a table.

void TDEWCursor::PackTable(const BOOL bRegenIdxs)
{
    TDEWException::CheckResult(DbiPackTable(fHDatabase, fHCursor, NULL, NULL, bRegenIdxs));
}


// Read a block of records.

UINT32 TDEWCursor::ReadBlock(const UINT32 iRecords, const pBYTE pBuf)
{
    UINT32 records = iRecords;

    TDEWException::CheckResult(DbiReadBlock(fHCursor, &records, pBuf));

    return records;
}


// Answer the record.

TDEWRecord& TDEWCursor::Record()
{
    return *fRecord;
}

const TDEWRecord& TDEWCursor::Record() const
{
    return *fRecord;
}


// Get the current number of records.

UINT32 TDEWCursor::RecordCount() const
{
    UINT32 iRecCount;

    TDEWException::CheckResult(DbiGetRecordCount(fHCursor, &iRecCount));

    return iRecCount;
}


// Regenerate an index.

void TDEWCursor::RegenIndex(const pcCHAR pszIndexName, const UINT16 iIdxNum, const pcCHAR pszIndexTagName)
{
    TDEWException::CheckResult(DbiRegenIndex(fHDatabase, fHCursor, NULL, NULL, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIdxNum));
}


// Regenerate all indexes.

void TDEWCursor::RegenIndexes()
{
    TDEWException::CheckResult(DbiRegenIndexes(fHCursor));
}


// Register callback function.

void TDEWCursor::RegisterCallBack(const CBType ecbType, const UINT32 iClientData, const UINT16 iCbBufLen, const pVOID pCbBuf, const pfDBICallBack pfCb)
{
    TDEWException::CheckResult(DbiRegisterCallBack(fHCursor, ecbType, iClientData, iCbBufLen, pCbBuf, pfCb));
}


// Release record level locks.

void TDEWCursor::RelRecordLock(const BOOL bAll)
{
    TDEWException::CheckResult(DbiRelRecordLock(fHCursor, bAll));
}


// Release table level locks.

void TDEWCursor::RelTableLock(const DBILockType eLockType, const BOOL bAll)
{
    TDEWException::CheckResult(DbiRelTableLock(fHCursor, bAll, eLockType));
}


// Remove the range limit.

void TDEWCursor::ResetRange()
{
    TDEWException::CheckResult(DbiResetRange(fHCursor));
}


// Flush all buffered changes.

void TDEWCursor::SaveChanges() const
{
    TDEWException::CheckResult(DbiSaveChanges(fHCursor));
}


// Get logical record number.

UINT32 TDEWCursor::SeqNo() const
{
    UINT32 iSeqNo;

    TDEWException::CheckResult(DbiGetSeqNo(fHCursor, &iSeqNo));

    return iSeqNo;
}


// Set field map.

void TDEWCursor::SetFieldMap(const UINT16 iFields, const pcFLDDesc paFldDesc)
{
    TDEWException::CheckResult(DbiSetFieldMap(fHCursor, iFields, (pFLDDesc) paFldDesc));
}


// Constrain the result set to the subset bounded by two keys.

void TDEWCursor::SetRange(const pcBYTE pKey1, const pcBYTE pKey2, const BOOL bKey1Incl, const BOOL bKey2Incl, const UINT16 iFields1, const UINT16 iLen1, const UINT16 iFields2, const UINT16 iLen2)
{
    TDEWException::CheckResult(DbiSetRange(fHCursor, TRUE, iFields1, iLen1, (pBYTE) pKey1, bKey1Incl, iFields2, iLen2, (pBYTE) pKey2, bKey2Incl));
}


// Reset cursor to beginning.

TDEWRecord& TDEWCursor::SetToBegin()
{
    TDEWException::CheckResult(DbiSetToBegin(fHCursor));

    return *fRecord;
}


// Position to a book-mark.

TDEWRecord& TDEWCursor::SetToBookMark(const pcBYTE pBookMark)
{
    TDEWException::CheckResult(DbiSetToBookMark(fHCursor, (pBYTE) pBookMark));

    return *fRecord;
}


// Set cursor to another cursor position.

TDEWRecord& TDEWCursor::SetToCursor(const TDEWCursor& cursor)
{
    TDEWException::CheckResult(DbiSetToCursor(fHCursor, cursor.fHCursor));

    return *fRecord;
}


// Reset cursor to ending.

TDEWRecord& TDEWCursor::SetToEnd()
{
    TDEWException::CheckResult(DbiSetToEnd(fHCursor));

    return *fRecord;
}


// Set key condition.

TDEWRecord& TDEWCursor::SetToKey(const TDEWRecord& record, const DBISearchCond eSearchCond, const UINT16 iFields, const UINT16 iLen)
{
    TDEWException::CheckResult(DbiSetToKey(fHCursor, eSearchCond, FALSE, iFields, iLen, record.fBuffer));

    return *fRecord;
}


// Position to physical record number.

TDEWRecord& TDEWCursor::SetToRecordNo(const UINT32 iRecNo)
{
    TDEWException::CheckResult(DbiSetToRecordNo(fHCursor, iRecNo));

    return *fRecord;
}


// Position to a logical record.

TDEWRecord& TDEWCursor::SetToSeqNo(const UINT32 iSeqNo)
{
    TDEWException::CheckResult(DbiSetToSeqNo(fHCursor, iSeqNo));

    return *fRecord;
}


// Sort the table.

void TDEWCursor::Sort(const UINT16 iSortFields, const pcUINT16 piFieldNum, const pcCHAR pszSortedName, const pcBOOL pbCaseInsensitive, const pcSORTOrder pSortOrder, pfSORTCompFn* ppfSortFn, const BOOL bRemoveDups)
{
    TDEWException::CheckResult(DbiSortTable(NULL, NULL, NULL, fHCursor, (pCHAR) pszSortedName, NULL, NULL, iSortFields, (pUINT16) piFieldNum, (pBOOL) pbCaseInsensitive, (pSORTOrder) pSortOrder, ppfSortFn, bRemoveDups, NULL, NULL));
}


// Change index order of access.

void TDEWCursor::SwitchToIndex(const pcCHAR pszIndexName, const BOOL bCurrRec, const UINT16 iIndexId, const pcCHAR pszTagName)
{
    TDEWException::CheckResult(DbiSwitchToIndex(&fHCursor, (pCHAR) pszIndexName, (pCHAR) pszTagName, iIndexId, bCurrRec));
}


// Undelete the current record.

TDEWRecord& TDEWCursor::UndeleteRecord()
{
    TDEWException::CheckResult(DbiUndeleteRecord(fHCursor));

    return *fRecord;
}


// Unlink detail from master.

void TDEWCursor::UnlinkDetail()
{
    TDEWException::CheckResult(DbiUnlinkDetail(fHCursor));
}


// Write a block of records.

UINT32 TDEWCursor::WriteBlock(const UINT32 iRecords, const pcBYTE pBuf)
{
    UINT32 records = iRecords;

    TDEWException::CheckResult(DbiWriteBlock(fHCursor, &records, (pBYTE) pBuf));

    return records;
}
