//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: DATABASE.CPP $
//  $Date: 1995/07/14 16:17:34 $


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructor.

TDEWDatabase::TDEWDatabase(const pcCHAR pszDbName, const pcCHAR pszDbType, const DBIOpenMode eOpenMode, const DBIShareMode eShareMode, const pcCHAR pszPassword, const UINT16 iOptFlds, const pcFLDDesc pOptFldDesc, const pcBYTE pOptParams)
  : fHDatabase(NULL)
{
    Open(pszDbName, pszDbType, eOpenMode, eShareMode, pszPassword, iOptFlds, pOptFldDesc, pOptParams);
}


// Destructor.

TDEWDatabase::~TDEWDatabase()
{
    if (IsOpen())
        Close();
}


// Get a persistent lock.

void TDEWDatabase::AcqPersistTableLock(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    TDEWException::CheckResult(DbiAcqPersistTableLock(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType));
}


// Add a new index.

void TDEWDatabase::AddIndex(const pcCHAR pszTableName, const pcCHAR pszDriverType, const IDXDesc& idxDesc, const pcCHAR pszKeyviolName)
{
    TDEWException::CheckResult(DbiAddIndex(fHDatabase, NULL, (pCHAR) pszTableName, (pCHAR) pszDriverType, (pIDXDesc) &idxDesc, (pCHAR) pszKeyviolName));
}


// Close database.

void TDEWDatabase::Close()
{
    TDEWException::CheckResult(DbiCloseDatabase(&fHDatabase));
    fHDatabase = NULL;
}


// Copy one table to another.

void TDEWDatabase::CopyTable(const pcCHAR pszSrcTblName, const pcCHAR pszDestTblName, const pcCHAR pszSrcDriverType, const BOOL bOverWrite)
{
    TDEWException::CheckResult(DbiCopyTable(fHDatabase, bOverWrite, (pCHAR) pszSrcTblName, (pCHAR) pszSrcDriverType, (pCHAR) pszDestTblName));
}


// Create a new table.

void TDEWDatabase::CreateTable(const CRTblDesc& crTblDesc, const BOOL bOverWrite)
{
    TDEWException::CheckResult(DbiCreateTable(fHDatabase, bOverWrite, (pCRTblDesc) &crTblDesc));
}


// Delete index.

void TDEWDatabase::DeleteIndex(const pcCHAR pszTableName, const pcCHAR pszIndexName, const UINT16 iIndexNum, const pcCHAR pszDriverType, const pcCHAR pszIndexTagName)
{
    TDEWException::CheckResult(DbiDeleteIndex(fHDatabase, NULL, (pCHAR) pszTableName, (pCHAR) pszDriverType, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIndexNum));
}


// Delete a table.

void TDEWDatabase::DeleteTable(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    TDEWException::CheckResult(DbiDeleteTable(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType));
}


// Get the database directory.

string TDEWDatabase::Directory(const BOOL bDefault) const
{
    DBIPATH directory;

    TDEWException::CheckResult(DbiGetDirectory(fHDatabase, bDefault, directory));

    return string(directory);
}


// Restructure a table.

void TDEWDatabase::DoRestructure(const UINT16 iTblDescCount, const pcCRTblDesc pTblDesc, const pcCHAR pszSaveAs, const pcCHAR pszKeyviolName, const pcCHAR pszProblemsName, const BOOL bAnalyzeOnly)
{
    TDEWException::CheckResult(DbiDoRestructure(fHDatabase, iTblDescCount, (pCRTblDesc) pTblDesc, (pCHAR) pszSaveAs, (pCHAR) pszKeyviolName, (pCHAR) pszProblemsName, bAnalyzeOnly));
}


// Delete all records.

void TDEWDatabase::EmptyTable(const pcCHAR pszTblName, const pcCHAR pszDriverType)
{
    TDEWException::CheckResult(DbiEmptyTable(fHDatabase, NULL, (pCHAR) pszTblName, (pCHAR) pszDriverType));
}


// Open a cursor on "family".

TDEWCursor TDEWDatabase::FamilyList(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenFamilyList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor);
}


// Return "fields" for a table.

TDEWCursor TDEWDatabase::FieldList(const pcCHAR pszTableName, const pcCHAR pszDriverType, const BOOL bPhyTypes)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenFieldList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, bPhyTypes, &hCursor));

    return TDEWCursor(hCursor);
}


// Open a cursor on "files".

TDEWCursor TDEWDatabase::FileList(const pcCHAR pszWild)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenFileList(fHDatabase, (pCHAR) pszWild, &hCursor));

    return TDEWCursor(hCursor);
}


// Form full name.

string TDEWDatabase::FullName(const pcCHAR pszTableName, const pcCHAR pszDriverType) const
{
    char fullName[DBIMAXTBLNAMELEN+1];

    TDEWException::CheckResult(DbiFormFullName(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, fullName));

    return string(fullName);
}


// Return "indexes" for a table.

TDEWCursor TDEWDatabase::IndexList(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenIndexList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor, FALSE);
}


// Create a temporary table.

TDEWCursor TDEWDatabase::InMemTable(const pcCHAR pszName, const UINT16 iFields, const pcFLDDesc pFldDesc)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiCreateInMemTable(fHDatabase, (pCHAR) pszName, iFields, (pFLDDesc) pFldDesc, &hCursor));

    return TDEWCursor(hCursor);
}


// Open a database.

void TDEWDatabase::Open(const pcCHAR pszDbName, const pcCHAR pszDbType, const DBIOpenMode eOpenMode, const DBIShareMode eShareMode, const pcCHAR pszPassword, const UINT16 iOptFlds, const pcFLDDesc pOptFldDesc, const pcBYTE pOptParams)
{
    if (IsOpen())
        Close();

    TDEWException::CheckResult(DbiOpenDatabase((pCHAR) pszDbName, (pCHAR) pszDbType, eOpenMode, eShareMode, (pCHAR) pszPassword, iOptFlds, (pFLDDesc) pOptFldDesc, (pBYTE) pOptParams, &fHDatabase));
}


// Pack a table.

void TDEWDatabase::PackTable(const pcCHAR pszTblName, const pcCHAR pszDriverType, const BOOL bRegenIdxs)
{
    TDEWException::CheckResult(DbiPackTable(fHDatabase, NULL, (pCHAR) pszTblName, (pCHAR) pszDriverType, bRegenIdxs));
}


// Execute a query.

TDEWCursor TDEWDatabase::Query(const pcCHAR pszQuery, const DBIQryLang eQryLang)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiQExecDirect(fHDatabase, eQryLang, (pCHAR) pszQuery, &hCursor));

    return TDEWCursor(hCursor);
}


// Regenerate an index.

void TDEWDatabase::RegenIndex(const pcCHAR pszTableName, const pcCHAR pszIndexName, const UINT16 iIdxNum, const pcCHAR pszDriverType, const pcCHAR pszIndexTagName)
{
    TDEWException::CheckResult(DbiRegenIndex(fHDatabase, NULL, (pCHAR) pszTableName, (pCHAR) pszDriverType, (pCHAR) pszIndexName, (pCHAR) pszIndexTagName, iIdxNum));
}


// Release a persistent lock.

void TDEWDatabase::RelPersistTableLock(const pcCHAR pszTableName, const pcCHAR pszTableType)
{
    TDEWException::CheckResult(DbiRelPersistTableLock(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszTableType));
}


// Rename table and family.

void TDEWDatabase::RenameTable(const pcCHAR pszOldName, const pcCHAR pszNewName, const pcCHAR pszDriverType)
{
    TDEWException::CheckResult(DbiRenameTable(fHDatabase, (pCHAR) pszOldName, (pCHAR) pszDriverType, (pCHAR) pszNewName));
}


// Return integrity checks.

TDEWCursor TDEWDatabase::RintList(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenRintList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor, FALSE);
}


// Return security descriptions.

TDEWCursor TDEWDatabase::SecurityList(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenSecurityList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor, FALSE);
}


// Set the database directory.

void TDEWDatabase::SetDirectory(const pcCHAR pszDir)
{
    TDEWException::CheckResult(DbiSetDirectory(fHDatabase, (pCHAR) pszDir));
}


// Sort a table.

void TDEWDatabase::Sort(const pcCHAR pszTableName, const UINT16 iSortFields, const pcUINT16 piFieldNum, const pcCHAR pszSortedName, const pcCHAR pszDriverType, const pcBOOL pbCaseInsensitive, const pcSORTOrder pSortOrder, pfSORTCompFn* ppfSortFn, const BOOL bRemoveDups)
{
    TDEWException::CheckResult(DbiSortTable(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, NULL, (pCHAR) pszSortedName, NULL, NULL, iSortFields, (pUINT16) piFieldNum, (pBOOL) pbCaseInsensitive, (pSORTOrder) pSortOrder, ppfSortFn, bRemoveDups, NULL, NULL));
}


// Open a cursor on "tables".

TDEWCursor TDEWDatabase::TableList(const BOOL bExtended, const BOOL bSystem, const pcCHAR pszWild)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenTableList(fHDatabase, bExtended, bSystem, (pCHAR) pszWild, &hCursor));

    return TDEWCursor(hCursor);
}


// Get local cursor count.

UINT16 TDEWDatabase::TableOpenCount(const pcCHAR pszTableName, const pcCHAR pszDriverType) const
{
    UINT16 iOpenCount;

    TDEWException::CheckResult(DbiGetTableOpenCount(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &iOpenCount));

    return iOpenCount;
}


// Create temporary table.

TDEWCursor TDEWDatabase::TempTable(const CRTblDesc& crTblDesc)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiCreateTempTable(fHDatabase, (pCRTblDesc) &crTblDesc, &hCursor));

    return TDEWCursor(hCursor);
}


// Return "checks" for a table.

TDEWCursor TDEWDatabase::VchkList(const pcCHAR pszTableName, const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenVchkList(fHDatabase, (pCHAR) pszTableName, (pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor);
}
