//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: STATEMNT.CPP $
//  $Date: 1995/07/14 16:17:34 $


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructors.

TDEWStatement::TDEWStatement()
  : fHDatabase(NULL),
    fHStatement(NULL)
{
}

TDEWStatement::TDEWStatement(TDEWDatabase& database)
  : fHDatabase(database.Handle()),
    fHStatement(NULL)
{
}

TDEWStatement::TDEWStatement(TDEWDatabase& database, const pcCHAR pszQuery, const DBIQryLang eQryLang)
  : fHDatabase(database.Handle()),
    fHStatement(NULL)
{
    Open(pszQuery, eQryLang);
}


// Destructor.

TDEWStatement::~TDEWStatement()
{
    if (IsOpen())
        Close();
}


// Close statement.

void TDEWStatement::Close()
{
    TDEWException::CheckResult(DbiQFree(&fHStatement));
    fHStatement = NULL;
}


// Execute query.

TDEWCursor TDEWStatement::Query() const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiQExec(fHStatement, &hCursor));

    return TDEWCursor(hCursor);
}

// Prepare a statement.

void TDEWStatement::Open(TDEWDatabase& database, const pcCHAR pszQuery, const DBIQryLang eQryLang)
{
    if (IsOpen())
        Close();

    fHDatabase = database.Handle();

    TDEWException::CheckResult(DbiQPrepare(fHDatabase, eQryLang, (pCHAR) pszQuery, &fHStatement));
}

void TDEWStatement::Open(const pcCHAR pszQuery, const DBIQryLang eQryLang)
{
    if (IsOpen())
        Close();

    TDEWException::CheckResult(DbiQPrepare(fHDatabase, eQryLang, (pCHAR) pszQuery, &fHStatement));
}


// Set parameters.

void TDEWStatement::SetParams(const UINT16 uFldDescs, const pcFLDDesc paFldDescs, const pcBYTE pRecBuff)
{
    TDEWException::CheckResult(DbiQSetParams(fHStatement, uFldDescs, (pFLDDesc) paFldDescs, (pBYTE) pRecBuff));
}
