//
//  Database Engine Wrapper
//  Copyright  1994, 1995
//  B. W. Bush
//  All rights reserved
//
//  $Revision: 2.0 $
//  $RCSfile: STRUCT.CPP $
//  $Date: 1995/07/14 16:14:53 $


// Definitions.

#include <dew.h>        // database engine wrapper


// Struct for defining the environ.

TDEWEnv::TDEWEnv()
{
    memset((DBIEnv*) this, 0, sizeof(DBIEnv));
}

TDEWEnv::TDEWEnv(const DBIEnv& dbiEnv)
  : DBIEnv(dbiEnv)
{
}

TDEWEnv::TDEWEnv(const pcCHAR workDir, const pcCHAR iniFile, const BOOL forceLocalInit, const pcCHAR lang, const pcCHAR clientName)
{
    memset((DBIEnv*) this, 0, sizeof(DBIEnv));

    strcpy(szWorkDir, workDir != NULL ? workDir : "");
    strcpy(szIniFile, iniFile != NULL ? iniFile : "");
    bForceLocalInit = forceLocalInit;
    strcpy(szLang, lang != NULL ? lang : "");
    strcpy(szClientName, clientName != NULL ? clientName : "");
}


// Field Descriptor.

TDEWFldDesc::TDEWFldDesc()
{
    memset((FLDDesc*) this, 0, sizeof(FLDDesc));
}

TDEWFldDesc::TDEWFldDesc(const FLDDesc& fldDesc)
  : FLDDesc(fldDesc)
{
}

TDEWFldDesc::TDEWFldDesc(const UINT16 fldNum, const pcCHAR name, const UINT16 fldType, const UINT16 subType, const INT16 units1, const INT16 units2)
{
    memset((FLDDesc*) this, 0, sizeof(FLDDesc));

    iFldNum = fldNum;
    strcpy(szName, name != NULL ? name : "");
    iFldType = fldType;
    iSubType = subType;
    iUnits1 = units1;
    iUnits2 = units2;
}


// Index description.

TDEWIdxDesc::TDEWIdxDesc()
{
    memset((IDXDesc*) this, 0, sizeof(IDXDesc));
}

TDEWIdxDesc::TDEWIdxDesc(const IDXDesc& idxDesc)
  : IDXDesc(idxDesc)
{
}

TDEWIdxDesc::TDEWIdxDesc(const DBIKEY keyFld, const UINT16 fldsInKey, const pcCHAR name, const UINT16 indexId, const pcCHAR tagName, const BOOL primary, const BOOL unique, const BOOL descending, const BOOL maintained, const BOOL caseInsensitive, const BOOL subset, const pcCHAR keyCond, const UINT16 blockSize)
{
    memset((IDXDesc*) this, 0, sizeof(IDXDesc));

    strcpy(szName, name != NULL ? name : "");
    iIndexId = indexId;
    strcpy(szTagName, tagName != NULL ? tagName : "");
    bPrimary = primary;
    bUnique = unique;
    bDescending = descending;
    bMaintained = maintained;
    bSubset = subset;
    bExpIdx = FALSE;
    iFldsInKey = fldsInKey;
    memcpy(aiKeyFld, keyFld, sizeof(DBIKEY));
    strcpy(szKeyCond, keyCond != NULL ? keyCond : "");
    bCaseInsensitive = caseInsensitive;
    iBlockSize = blockSize;
}

TDEWIdxDesc::TDEWIdxDesc(const pcCHAR keyExp, const UINT16 keyExpType, const pcCHAR name, const UINT16 indexId, const pcCHAR tagName, const BOOL primary, const BOOL unique, const BOOL descending, const BOOL maintained, const BOOL caseInsensitive, const BOOL subset, const pcCHAR keyCond , const UINT16 blockSize)
{
    memset((IDXDesc*) this, 0, sizeof(IDXDesc));

    strcpy(szName, name != NULL ? name : "");
    iIndexId = indexId;
    strcpy(szTagName, tagName != NULL ? tagName : "");
    bPrimary = primary;
    bUnique = unique;
    bDescending = descending;
    bMaintained = maintained;
    bSubset = subset;
    bExpIdx = TRUE;
    iFldsInKey = 0;
    iKeyExpType = keyExpType;
    strcpy(szKeyExp, keyExp != NULL ? keyExp : "");
    strcpy(szKeyCond, keyCond != NULL ? keyCond : "");
    bCaseInsensitive = caseInsensitive;
    iBlockSize = blockSize;
}


// Val Check structure.

TDEWVchkDesc::TDEWVchkDesc()
{
    memset((VCHKDesc*) this, 0, sizeof(VCHKDesc));
}

TDEWVchkDesc::TDEWVchkDesc(const VCHKDesc& vchkDesc)
  : VCHKDesc(vchkDesc)
{
}

TDEWVchkDesc::TDEWVchkDesc(const UINT16 fldNum)
{
    memset((VCHKDesc*) this, 0, sizeof(VCHKDesc));

    iFldNum = fldNum;
    bRequired = FALSE;
    bHasMinVal = FALSE;
    bHasMaxVal = FALSE;
    bHasDefVal = FALSE;
    strcpy(szPict, "");
    elkupType = lkupNONE;
    strcpy(szLkupTblName, "");
}

void TDEWVchkDesc::SetRequired()
{
    bRequired = TRUE;
}

void TDEWVchkDesc::SetMinVal(const pcBYTE minVal)
{
    bHasMinVal = TRUE;
    memcpy(aMinVal, minVal, sizeof(DBIVCHK));
}

void TDEWVchkDesc::SetMinValToday()
{
    bHasMinVal = TODAYVAL;
}

void TDEWVchkDesc::SetMaxVal(const pcBYTE maxVal)
{
    bHasMaxVal = TRUE;
    memcpy(aMaxVal, maxVal, sizeof(DBIVCHK));
}

void TDEWVchkDesc::SetMaxValToday()
{
    bHasMaxVal = TODAYVAL;
}

void TDEWVchkDesc::SetDefVal(const pcBYTE defVal)
{
    bHasDefVal = TRUE;
    memcpy(aDefVal, defVal, sizeof(DBIVCHK));
}

void TDEWVchkDesc::SetDefValToday()
{
    bHasDefVal = TODAYVAL;
}

void TDEWVchkDesc::SetPict(const pcCHAR pict)
{
    strcpy(szPict, pict);
}

void TDEWVchkDesc::SetLkup(const LKUPType lkupType, const pcCHAR lkupTblName)
{
    elkupType = lkupType;
    strcpy(szLkupTblName, lkupTblName);
}


// Ref Integrity Desc.

TDEWRintDesc::TDEWRintDesc()
{
    memset((RINTDesc*) this, 0, sizeof(RINTDesc));
}

TDEWRintDesc::TDEWRintDesc(const RINTDesc& rintDesc)
  : RINTDesc(rintDesc)
{
}

TDEWRintDesc::TDEWRintDesc(const UINT16 rintNum, const pcCHAR rintName, const pcCHAR tblName, const DBIKEY thisTabFld, const DBIKEY othTabFld, const UINT16 fldCount, const RINTQual modOp, const RINTQual delOp, const RINTType type)
{
    memset((RINTDesc*) this, 0, sizeof(RINTDesc));

    iRintNum = rintNum;
    strcpy(szRintName, rintName);
    eType = type;
    strcpy(szTblName, tblName);
    eModOp = modOp;
    eDelOp = delOp;
    iFldCount = fldCount;
    memcpy(aiThisTabFld, thisTabFld, sizeof(DBIKEY));
    memcpy(aiOthTabFld, othTabFld, sizeof(DBIKEY));
}


// Security description.

TDEWSecDesc::TDEWSecDesc()
{
    memset((SECDesc*) this, 0, sizeof(SECDesc));
}

TDEWSecDesc::TDEWSecDesc(const SECDesc& secDesc)
  : SECDesc(secDesc)
{
}

TDEWSecDesc::TDEWSecDesc(const UINT16 secNum, const PRVType prvFld[DBIMAXFLDSINSEC], const pcCHAR password, const PRVType prvTable, const UINT16 famRights)
{
    memset((SECDesc*) this, 0, sizeof(SECDesc));

    iSecNum = secNum;
    eprvTable = prvTable;
    iFamRights = famRights;
    strcpy(szPassword, password);
    memcpy(aprvFld, prvFld, sizeof(PRVType) * DBIMAXFLDSINSEC);
}


// Create/Restruct Table descr.

TDEWCrTblDesc::TDEWCrTblDesc()
{
    memset((CRTblDesc*) this, 0, sizeof(CRTblDesc));
}

TDEWCrTblDesc::TDEWCrTblDesc(const CRTblDesc& crTblDesc)
  : CRTblDesc(crTblDesc)
{
}

TDEWCrTblDesc::TDEWCrTblDesc(const pcCHAR tblName, const UINT16 fldCount, const pcFLDDesc fldDesc, const UINT16 idxCount, const pcIDXDesc idxDesc, const UINT16 valChkCount, const pcVCHKDesc vchkDesc, const UINT16 rintCount, const pcRINTDesc rintDesc, const UINT16 secRecCount, const pcSECDesc secDesc, const pcCHAR password, const pcCHAR tblType)
{
    memset((CRTblDesc*) this, 0, sizeof(CRTblDesc));

    strcpy(szTblName, tblName);
    strcpy(szTblType, tblType != NULL ? tblType : "");
    strcpy(szPassword, password != NULL ? password : "");
    bProtected = password != NULL;
    iFldCount = fldCount;
    pfldDesc = (pFLDDesc) fldDesc;
    iIdxCount = idxCount;
    pidxDesc = (pIDXDesc) idxDesc;
    iSecRecCount = secRecCount;
    psecDesc = (pSECDesc) secDesc;
    iValChkCount = valChkCount;
    pvchkDesc = (pVCHKDesc) vchkDesc;
    iRintCount = rintCount;
    printDesc = (pRINTDesc) rintDesc;
    iOptParams = 0;
}
